from base_st_classes import *

class CreateSuite(NFSSuite):
    """Test operation 6: CREATE

    FIXME: Add attribute directory and named attribute testing.

    Equivalence partitioning:

    Input Condition: current filehandle
        Valid equivalence classes:
            dir(10)
        Invalid equivalence classes:
            not dir(11)
            no filehandle(12)
    Input Condition: type
        Valid equivalence classes:
            link(20)
            blockdev(21)
            chardev(22)
            socket(23)
            FIFO(24)
            directory(25)
        Invalid equivalence classes:
            regular file(26)
    Input Condition: name
        Valid equivalence classes:
            legal name(30)
        Invalid equivalence classes:
            zero length(31)
    Input Condition: fattr.attrmask
        Valid equivalence classes:
            valid attrmask(40)
        Invalid equivalence classes:
            invalid attrmask(41) (FATTR4_*_SET)
    Input Condition: fattr.attr_vals
        Valid equivalence classes:
            valid attribute value(50)
        Invalid equivalence classes:
            valid attribute value(51)

    """
    __pychecker__ = 'no-classattr'
    
    def setUp(self):
        NFSSuite.setUp(self)
        self.obj_name = "object1"

        self.lookup_dir_ops = self.ncl.lookup_path(self.tmp_dir)

    def getDefaultAttr(self):
        attr = {}
        if nfs4lib.OWNER:
            attr[FATTR4_OWNER] = nfs4lib.OWNER
        if nfs4lib.OWNER_GROUP:
            attr[FATTR4_OWNER_GROUP] = nfs4lib.OWNER_GROUP
        return nfs4lib.dict2fattr(attr)

    def tearDown(self):
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    #
    # Testcases covering valid equivalence classes.
    #
    def testLink(self):
        """CREATE (symbolic) link

        Covered valid equivalence classes: 10, 20, 30, 40, 50
        """
        ops = [self.ncl.putrootfh_op()] +  self.lookup_dir_ops
        objtype = createtype4(type=NF4LNK, linkdata="/etc/X11")
        createop = self.ncl.create_op(objtype, self.obj_name, self.getDefaultAttr())
        ops.append(createop)
        res = self.ncl.do_ops(ops)

        if res.status == NFS4ERR_BADTYPE:
            self.info_message("links not supported")
        self.assert_status(res, [NFS4_OK, NFS4ERR_BADTYPE])

        # Try again, should get NFS4ERR_EXIST
        if res.status == NFS4_OK:
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_EXIST])

    def testBlock(self):
        """CREATE a block device

        Covered valid equivalence classes: 10, 21, 30, 40, 50
        """
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
        
        devdata = specdata4(1, 2)
        objtype = createtype4(type=NF4BLK, devdata=devdata)
        createop = self.ncl.create_op(objtype, self.obj_name, self.getDefaultAttr())
        ops.append(createop)

        # FIXME: Maybe try to create block and character devices as root.
        res = self.ncl.do_ops(ops)
        if res.status == NFS4ERR_BADTYPE:
            self.info_message("blocks devices not supported")
        elif res.status == NFS4ERR_PERM:
            self.info_message("not permitted")
        else:
            self.assert_status(res, [NFS4_OK, NFS4ERR_BADTYPE, NFS4ERR_PERM])

        # Try again, should get NFS4ERR_EXIST
        if res.status == NFS4_OK:
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_EXIST])

    def testChar(self):
        """CREATE a character device

        Covered valid equivalence classes: 10, 22, 30, 40, 50
        """
        ops = [self.ncl.putrootfh_op()] +  self.lookup_dir_ops
        devdata = specdata4(1, 2)
        objtype = createtype4(type=NF4CHR, devdata=devdata)
        createop = self.ncl.create_op(objtype, self.obj_name, self.getDefaultAttr())
        ops.append(createop)

        res = self.ncl.do_ops(ops)
        if res.status == NFS4ERR_BADTYPE:
            self.info_message("character devices not supported")
        elif res.status == NFS4ERR_PERM:
            self.info_message("not permitted")
        else:
            self.assert_status(res, [NFS4_OK, NFS4ERR_BADTYPE, NFS4ERR_PERM])

        # Try again, should get NFS4ERR_EXIST
        if res.status == NFS4_OK:
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_EXIST])

    def testSocket(self):
        """CREATE a socket

        Covered valid equivalence classes: 10, 23, 30, 40, 50
        """
        ops = [self.ncl.putrootfh_op()] +  self.lookup_dir_ops
        objtype = createtype4(type=NF4SOCK)
        createop = self.ncl.create_op(objtype, self.obj_name, self.getDefaultAttr())
        ops.append(createop)

        res = self.ncl.do_ops(ops)
        if res.status == NFS4ERR_BADTYPE:
            self.info_message("sockets not supported")
        self.assert_status(res, [NFS4_OK, NFS4ERR_BADTYPE])

        # Try again, should get NFS4ERR_EXIST
        if res.status == NFS4_OK:
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_EXIST])


    def testFIFO(self):
        """CREATE a FIFO

        Covered valid equivalence classes: 10, 24, 30, 40, 50
        """
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
        objtype = createtype4(type=NF4FIFO)
        createop = self.ncl.create_op(objtype, self.obj_name, self.getDefaultAttr())
        ops.append(createop)

        res = self.ncl.do_ops(ops)
        if res.status == NFS4ERR_BADTYPE:
            self.info_message("FIFOs not supported")
        self.assert_status(res, [NFS4_OK, NFS4ERR_BADTYPE])

        # Try again, should get NFS4ERR_EXIST
        if res.status == NFS4_OK:
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_EXIST])

    def testDir(self):
        """CREATE a directory

        Covered valid equivalence classes: 10, 25, 30, 40, 50
        """
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
        objtype = createtype4(type=NF4DIR)
        createop = self.ncl.create_op(objtype, self.obj_name, self.getDefaultAttr())
        ops.append(createop)

        res = self.ncl.do_ops(ops)
        if res.status == NFS4ERR_BADTYPE:
            self.info_message("directories not supported!")
        self.assert_status(res, [NFS4_OK, NFS4ERR_BADTYPE])

        # Try again, should get NFS4ERR_EXIST
        if res.status == NFS4_OK:
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_EXIST])

    #
    # Testcases covering invalid equivalence classes.
    #
    def testFhNotDir(self):
        """CREATE should fail with NFS4ERR_NOTDIR if (cfh) is not dir

        Covered invalid equivalence classes: 11
        """
        for path in [ self.regfile,
                      self.blockfile,
                      self.charfile,
                      self.linkfile,
                      self.socketfile,
                      self.fifofile ]:
            lookupops = self.ncl.lookup_path(path)
            ops = [self.ncl.putrootfh_op()] + lookupops
            objtype = createtype4(type=NF4DIR)
            createop = self.ncl.create_op(objtype, self.obj_name,
                                          self.getDefaultAttr())
            ops.append(createop)
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_NOTDIR])

    def testNoFh(self):
        """CREATE should fail with NFS4ERR_NOFILEHANDLE if no (cfh)

        Covered invalid equivalence classes: 12
        """
        objtype = createtype4(type=NF4DIR)
        createop = self.ncl.create_op(objtype, self.obj_name,
                                      self.getDefaultAttr())
        res = self.ncl.do_ops([createop])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])

    def testZeroLengthName(self):
        """CREATE with zero length name should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 31
        """
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
        objtype = createtype4(type=NF4DIR)
        createop = self.ncl.create_op(objtype, "", self.getDefaultAttr())
        ops.append(createop)

        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4ERR_INVAL])

    def testRegularFile(self):
        """CREATE should fail with NFS4ERR_BADTYPE for regular files

        (see p245 of RPC 3530)
        Covered invalid equivalence classes: 26
        """
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops

        objtype = createtype4(type=NF4REG)
        createop = self.ncl.create_op(objtype, self.obj_name,
                                      self.getDefaultAttr())
        ops.append(createop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4ERR_BADTYPE])

    def testInvalidAttrmask(self):
        """CREATE should fail with NFS4ERR_INVAL on invalid attrmask

        Covered invalid equivalence classes: 41

        Comments: We are using a read-only attribute on CREATE, which
        should return NFS4ERR_INVAL. 
        """
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
        objtype = createtype4(type=NF4DIR)
        attrs = nfs4lib.dict2fattr({FATTR4_LINK_SUPPORT: TRUE})
        createop = self.ncl.create_op(objtype, self.obj_name, attrs)
        ops.append(createop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4ERR_INVAL])

    def xtestInvalidAttributes(self):
        """CREATE should fail with NFS4ERR_XDR on invalid attr_vals

        Covered invalid equivalence classes: 51

        Comments: BADXDR should take precedence over NOTSUPP; BADXDR
        should be returned even if the server does not support the attribute

        (FRED) - why?
        
        (FRED) - Right now it is fairly hard to get the python code to
        send an incorrect value.  Work on this.
        """
        print
        supported = self.supportedAttrs(self.dirfile)
        attrbitnum_dict = nfs4lib.get_attrbitnum_dict()
        for (attr,value) in self.ncl.writable_attrs:
            bitnum = attrbitnum_dict[attr]
            if not (supported & (2**bitnum)):
                print attr
                '''
                ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
                objtype = createtype4(type=NF4DIR)
                attrs = nfs4lib.dict2fattr({bitnum: 10})
                createop = self.ncl.create_op(objtype, self.obj_name, attrs)
                ops.append(createop)
                res = self.ncl.do_ops(ops)
                self.assert_status(res, [NFS4ERR_ATTRNOTSUPP])
                '''
                
    def testUnsupportedAttributes(self):
        """CREATE should fail with NFS4ERR_ATTRNOTSUPP on unsupported attrs
        """
        supported = self.supportedAttrs(self.dirfile)
        attrbitnum_dict = nfs4lib.get_attrbitnum_dict()
        for (attr,value) in self.ncl.writable_attrs:
            bitnum = attrbitnum_dict[attr]
            if not (supported & (2**bitnum)):
                ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
                objtype = createtype4(type=NF4DIR)
                attrs = nfs4lib.dict2fattr({bitnum: value})
                createop = self.ncl.create_op(objtype, self.obj_name, attrs)
                ops.append(createop)
                res = self.ncl.do_ops(ops)
                self.assert_status(res, [NFS4ERR_ATTRNOTSUPP])

    #
    # Extra tests.
    #
    def _do_create(self, name):
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
        objtype = createtype4(type=NF4DIR)
        createop = self.ncl.create_op(objtype, name, self.getDefaultAttr())
        ops.append(createop)

        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4_OK, NFS4ERR_INVAL])

    def testDots(self):
        """CREATE with . or .. should succeed or return NFS4ERR_INVAL

        Extra test

        Servers supporting . and .. in file names should return NFS4_OK. Others
        should return NFS4ERR_INVAL. NFS4ERR_EXIST should not be returned.
        """
        self._do_create(".")
        self._do_create("..")

    def testSlash(self):
        """CREATE with "/" in filename should succeed or return NFS4ERR_INVAL

        Extra test

        Make sure / in file names are not treated as directory
        separator. Servers supporting "/" in file names should return
        NFS4_OK. Others should return NFS4ERR_INVAL. NFS4ERR_EXIST
        should not be returned.
        """
        # Great idea. Try this:
        # tmp
        # |-- "gazonk/foo.c"
        # `-- gazonk
        #     `--foo.c
        #
        # /tmp/gazonk/foo.c is created by test_tree.py. 
        
        testname = "gazonk/foo.c"
        # Try to create "gazonk/foo.c"
        self._do_create(testname)

    def testNamingPolicy(self):
        """CREATE should obey OPEN file name creation policy

        Extra test
        """
        self.init_connection()

        try:
            (x, rejected_names_open) = self.try_file_names(creator=self.create_via_open)
            
            (x, rejected_names_create) = self.try_file_names(creator=self.create_via_create)
            self.failIf(rejected_names_open != rejected_names_create,
                        "CREATE does not obey OPEN naming policy")
        except SkipException, e:
            self.skip(e)

    #
    # Testing various error responses
    #
    def testLongName(self):
        """CREATE should fail with NFS4ERR_NAMETOOLONG with long filenames
        """
        # Create a very long name
        name = "a"*512
        ops = [self.ncl.putrootfh_op()] + self.lookup_dir_ops
        objtype = createtype4(type=NF4DIR)
        createop = self.ncl.create_op(objtype, name, self.getDefaultAttr())
        ops.append(createop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4ERR_NAMETOOLONG])


