from base_st_classes import *

class LockuSuite(NFSSuite):
    """Test operation 14: LOCKU
    """

    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None
        self.init_connection()

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    #
    # Tests that should work
    #
    def testFile(self):
        """LOCKU a regular file
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        self.ncl.lock_test(self.fh)
        self.ncl.unlock_file(self.fh, lockid, 1)
        
    #
    # Tests that may or may not work depending on server
    #
    def testUnlocked(self):
        """LOCKU on an unlocked area should work or return NFS4ERR_LOCK_RANGE
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid, 100, 100)
        self.ncl.unlock_file(self.fh, lockid, 1, 0, 50,
                             error=[NFS4_OK, NFS4ERR_LOCK_RANGE])
        
    def testOverlap(self):
        """LOCKU on an overlapped range should work or return NFS4ERR_LOCK_RANGE
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid, 100, 100)
        lockid = self.ncl.unlock_file(self.fh, lockid, 1, 50, 100,
                             error=[NFS4_OK, NFS4ERR_LOCK_RANGE])
        if not lockid: return

        # Make sure we have a single lock from 150 to 200
        res = self.ncl.lock_test(self.fh, 125, 50)
        obj = res.resarray[-1].arm.arm
        self.assertEqual(obj.offset,150,"Offset not 150")
        self.assertEqual(obj.length,50,"Length not 50")

    def test32bitRange(self):
        """LOCKU range over 32 bits may not be supported
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        lockid = self.ncl.unlock_file(self.fh, lockid, 1, 0, 0xffffffffffff,
                      error=[NFS4_OK, NFS4ERR_BAD_RANGE, NFS4ERR_LOCK_RANGE])

    #
    # Tests for various error conditions
    #
    def testZeroLen(self):
        """LOCKU with len=0 should return NFS4ERR_INVAL
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        lockid = self.ncl.unlock_file(self.fh, lockid, 1, len=0,
                                      error=[NFS4ERR_INVAL])

    def testLenTooLong(self):
        """LOCKU with offset+len overflow should return NFS4ERR_INVAL
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        lockid = self.ncl.unlock_file(self.fh, lockid, 1,
                                      100, 0xfffffffffffffffe,
                                      error=[NFS4ERR_INVAL])

    def testNoCfh(self):
        """LOCKU with no (cfh) should return NFS4ERR_NOFILEHANDLE
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        
        unlockop = self.ncl.locku_op(READ_LT, 1, lockid, 0, 0xffffffffffffffff)
        res = self.ncl.do_ops([unlockop])
        self.assert_status(res,[NFS4ERR_NOFILEHANDLE])
      
    def testBadSeqid(self):
        """LOCKU with a bad sequence should return NFS4ERR_BAD_SEQID
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        lockid = self.ncl.unlock_file(self.fh, lockid, 2,
                                      error=[NFS4ERR_BAD_SEQID])

    def testOldStateid(self):
        """LOCKU with old stateid should return NFS4ERR_OLD_STATEID
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        self.ncl.unlock_file(self.fh, lockid, 1)
        
        # Try to unlock again with old stateid
        self.ncl.unlock_file(self.fh, lockid, 2, error=[NFS4ERR_OLD_STATEID])

    def testBadStateid(self):
        """LOCKU should return NFS4ERR_BAD_STATEID if use a bad id
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = stateid4(0, "")
        self.ncl.unlock_file(self.fh, lockid, 1, error=[NFS4ERR_BAD_STATEID])

    def testTimedoutUnlock(self):
        """LOCKU: Try to unlock file after timed out 

        This returns BAD_STATEID, is this correct?
        """
        #FRED - correct return? is NFS4_OK ok too?
        # This causes kernal message
        # NFSD: find_stateid: ERROR: no state flag
        sleeptime = self.getLeaseTime() * 3 / 2
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        self.sleep(sleeptime)
        self.ncl.unlock_file(self.fh, lockid, 1,
                             error=[NFS4ERR_BAD_STATEID, NFS4_OK])
