from base_st_classes import *

class OpenconfirmSuite(NFSSuite):
    """Test operation 20: OPEN_CONFIRM
    """
    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None
        self.init_connection()

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    def _open(self,dir,name):
        ops = [self.ncl.putrootfh_op()]+self.ncl.lookup_path(dir)
        ops.append(self.ncl.open(name))
        ops.append(self.ncl.getfh_op())
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        fh = res.resarray[-1].arm.arm.object
        stateid = res.resarray[-2].arm.arm.stateid
        return (fh,stateid)

    #
    # Tests that should work
    #
    
    def testConfirmCreate(self):
        """OPEN_CONFIRM after a normal open create
        """
        # Create the file and confirm
        self.fh, self.stateid = self.ncl.create_confirm()

    def testConfirmOpen(self):
        """OPEN_CONFIRM after a noncreate open
        """
        # Open and confirm a file
        dir = self.regfile[0:-1]
        name = self.regfile[-1]
        self.fh, self.stateid = self.ncl.open_confirm(dir,name)


    #
    # Tests of various errors
    #

    def testConfirmCreate(self):
        """OPEN_CONFIRM done twice after a normal open should return NFS4ERR_BAD_STATEID
        """
        # Create the file and confirm
        self.fh, self.stateid = self.ncl.create_confirm()

        # Now confirm again
        ops = [self.ncl.putfh_op(self.fh)]
        confirmop = self.ncl.open_confirm_op(self.stateid,self.ncl.get_seqid())
        ops.append(confirmop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_BAD_STATEID])
        
    def testNoCfh(self):
        """OPEN_CONFIRM with no cfh should return NFS4ERR_NOFILEHANDLE
        """
        # Open file to get a valid stateid
        self.fh, self.stateid = self._open(self.regfile[:-1], self.regfile[-1])

        # Call open_confirm without setting cfh
        ops = [self.ncl.open_confirm_op(self.stateid, self.ncl.get_seqid(False))]
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_NOFILEHANDLE])

    def testDir(self):
        """OPEN_CONFIRM of a dir should return NFS4ERR_ISDIR
        """
        # Open file to get a valid stateid
        self.fh, self.stateid = self._open(self.regfile[:-1], self.regfile[-1])

        # Call open_confirm without a dir in cfh
        ops = [self.ncl.putrootfh_op()] + self.ncl.lookup_path(self.dirfile)
        ops.append(self.ncl.open_confirm_op(self.stateid,
                                            self.ncl.get_seqid(False)))
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_ISDIR])

    def testDev(self):
        """OPEN_CONFIRM of a device should return NFS4ERR_INVAL
        """
        # Open file to get a valid stateid
        self.fh, self.stateid = self._open(self.regfile[:-1], self.regfile[-1])

        # Call open_confirm without a dir in cfh
        ops = [self.ncl.putrootfh_op()] + self.ncl.lookup_path(self.charfile)
        ops.append(self.ncl.open_confirm_op(self.stateid, self.ncl.get_seqid(False)))
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_INVAL])

    def testBadSeqid(self):
        """OPEN_CONFIRM with a bad seqid should return NFS4ERR_BAD_SEQID
        """
        # Open file
        self.fh, self.stateid = self._open(self.regfile[:-1], self.regfile[-1])

        # Call open_confirm with wrong seqid
        seqid = self.ncl.get_seqid() + 1
        ops = [self.ncl.putfh_op(self.fh)]
        confirmop = self.ncl.open_confirm_op(self.stateid, seqid)
        ops.append(confirmop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_BAD_SEQID])

    def testBadStateid(self):
        """OPEN_CONFIRM with a bad state should return NFS4ERR_BAD_STATEID
        """
        # Open file
        self.fh, self.stateid = self._open(self.regfile[:-1], self.regfile[-1])

        # Call open_confirm with all 0 stateid
        stateid = stateid4(0, "")
        ops = [self.ncl.putfh_op(self.fh)]
        confirmop = self.ncl.open_confirm_op(stateid, 0)
        ops.append(confirmop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_BAD_STATEID])

 
